// @dart=2.12

import 'dart:convert';
import 'dart:io';

import '../models/app_version.dart';
// ignore: import_of_legacy_library_into_null_safe
import '../release.dart';
import 'base_command.dart';

class AppCommand extends BaseCommand {
  static const _kIosExtensionBumpName = 'ios-extension-bump';
  static const _kTagName = 'tag';

  @override
  String get description =>
      'Specify to update version for a Flutter application.';

  @override
  String get name => 'app';

  AppCommand() {
    argParser.addFlag(
      _kIosExtensionBumpName,
      help: 'Whether to specially handle iOS bumping. Default to true',
      negatable: true,
      defaultsTo: true,
    );

    argParser.addFlag(
      _kTagName,
      help: 'Create a tag in local.\n* Take note: You must in master branch.',
      defaultsTo: false,
      negatable: false,
    );
  }

  @override
  Future<void> run() async {
    if (argResults == null || globalResults == null) {
      throw 'Something when wrong when attempting to parse argument result.';
    }

    if (globalResults!.wasParsed(kBumpTypeName)) {
      await formatFiles();
      await _updateVersionAsync();
    } else if (argResults!.wasParsed(_kTagName)) {
      await createTagAsync(
        gitDir: await getGitDir(Directory.current.path),
        pubspecYamlPath: projectRootPath,
      );
    }
    
    print('Done');
  }

  Future<void> _updateVersionAsync() async {
    final currentVersion = await getProjectVersion(projectRootPath);
    await bumpVersionAsync(projectPath: projectRootPath);
    final newVersion = await getProjectVersion(projectRootPath);

    print('Version updated from $currentVersion to $newVersion.');

    if (argResults![_kIosExtensionBumpName]) {
      // This is needed as flutter 1.20.X breaks the placeholder in app
      // extension project: https://github.com/flutter/flutter/issues/63551.
      print('Start modifying iOS version...');

      final flutterProjectDir = Directory.current;
      Directory.current = 'ios';
      final version = AppVersion.parse(newVersion);
      Process.runSync(
        'agvtool',
        'new-version -all ${version.versionNumber}'.split(' '),
        stdoutEncoding: utf8,
        runInShell: runInShell,
      );
      Process.runSync(
        'agvtool',
        'new-marketing-version ${version.buildName}'.split(' '),
        stdoutEncoding: utf8,
        runInShell: runInShell,
      );
      Directory.current = flutterProjectDir;
      print('Done modifying iOS version...');
    }
  }
}
