import 'package:args/args.dart';
import 'package:args/command_runner.dart';
import 'package:console/console.dart';

import 'commands/app_command.dart';
import 'commands/package_command.dart';

const kBumpTypes = ['major', 'minor', 'patch'];

const kBumpTypeName = 'type';
const kProjectRootName = 'project-root';
const kDefaultProjectRoot = '.';

Future<void> main(List<String> args) async {
  final commandRunner = CommandRunner(
    'Release script',
    'Flutter application and package release script',
  )
    ..addCommand(AppCommand())
    ..addCommand(PackageCommand());

  commandRunner.argParser
    ..addOption(
      kBumpTypeName,
      allowed: kBumpTypes,
      abbr: 't',
      help: '''
    Consider 1.2.1 as current version. 
    - \'major\' will increment the version to 2.0.0. 
    - \'minor\' will increment the version to 1.3.1. 
    - \'patch\' will increment the version to 1.2.2. 
    ''',
    )
    ..addOption(
      kProjectRootName,
      defaultsTo: kDefaultProjectRoot,
      help: '''
      The project root path that contains pubspec.yaml.
      ''',
    );
  try {
    await commandRunner.run(args);
  } catch (ex) {
    handleException(ex, commandRunner.argParser);
  }
}

void handleException(dynamic ex, ArgParser parser) {
  final textPen = TextPen();
  textPen.red();
  if (ex is String) {
    textPen.text('$ex\n');
  } else if (ex is FormatException) {
    textPen.text('${ex.message}\n');
  } else {
    print(ex);
  }

  textPen.white();

  textPen.text('\n');
  textPen.text('${parser.usage}\n');
  textPen.print();
}
